/*Style for control bar section*/

.control-bar-section{
    display: flex;
    gap: 1rem;
    padding: 1rem 0;
    align-items: end;
    margin-top: 1rem;
}

.ssb-input.file-path-input{
    max-width: 1100px;
}

.ssb-btn.secondary-btn.file-open-button,.ssb-btn.secondary-btn.file-save-button{
    margin-top: 1.5rem;
}

.jsx-4017309047.tab-container.ssb-tabs{
    display: flex;
    flex-direction: row;
    border-bottom: 2px solid #274247;
}

.jsx-3468109796.tab.workspace-tab{
    font-size: 1rem;
    font-weight: bold;
    align-items: center;
    background-color: transparent;
    border: none;
    color: inherit;
    font-family: "Roboto", sans-serif !important;
    font-stretch: normal;
    cursor: pointer;
    display: flex;
    justify-content: center;
    line-height: 1.25;
    outline: none;
    padding: 20px 0;
    position: relative;
    text-align: center;
    text-decoration: none;
    transition: background 0.2s;
    width: fit-content;
}

.jsx-3468109796.tab.workspace-tab:not(:last-of-type){
  margin-right: 40px;
}

.jsx-4017309047.tab-container.ssb-tabs >.tab.jsx-3468109796:last-of-type{
  border: unset;
}
.jsx-3468109796.tab.workspace-tab::before{
  background: #274247;
  bottom: 0;
  content: "";
  display: block;
  height: 0;
  left: 0;
  position: absolute;
  transition: all 0.2s;
  width: 100%;
}

.jsx-3468109796.tab.workspace-tab:hover::before{
  height: 4px;
}
.jsx-3468109796.tab.workspace-tab:focus::before{
  height: 4px;
}
.jsx-4017309047.tab-container.ssb-tabs > .jsx-3468109796.tab.workspace-tab.tab--selected {
    border-bottom: 4px solid #1a9d49;
}

.jsx-4017309047.tab-container.ssb-tabs > .jsx-3468109796.tab.workspace-tab.tab--selected:hover::before{
  height: unset;
}
@media screen and (min-width: 800px){
    .jsx-4017309047.tab-container.ssb-tabs > .tab--selected.jsx-3468109796, .tab.jsx-3468109796:last-of-type.tab--selected {
        border: none;
        border-bottom: 4px solid #1a9d49;
    }
}

#alerts-section{
  display: flex;
  gap: 1rem;
  padding-top: 1rem;
  width: 100%;
}

#alerts-section > div.fade.ssb-alert.alert.alert-warning.alert-dismissible.show {
  border: 6px solid #ffecb5;

}
#alerts-section > div.fade.ssb-alert.alert.alert-warning.alert-dismissible.show > .alert_link{
  padding: 0;
  margin: 0;
  margin-bottom: 1rem;
}
#alerts-section > div.fade.ssb-alert.alert.alert-success.alert-dismissible.show{
  border: 6px solid #badbcc;
}

#alerts-section > div.fade.ssb-alert.alert.alert-warning.alert-dismissible.show > .alert_list{
  margin: 0;
  margin-top: 1rem;
}

.button-section{
  display: flex;
  gap: 1rem;
}

@media only screen and (max-width: 1200px) {
  .control-bar-section{
    flex-wrap: wrap;
  }
}
