/*Style for app containers*/

.app-wrapper{
    max-width: 1900px;
    width: 100%;
    margin: 1rem auto
}

.main-content-app{
    margin: 1.5rem 12.25rem;
}

.language-footer{
    margin: 1.5rem 12.25rem;
}
